@@ -36,8 +36,6 @@
 #include "status.h"
 
 
-
-
 #define CRCINIT 0xB704CE
 #define CRCPOLY 0X864CFB
 #define CRCUPDATE(a,c) do {						    \
@@ -89,12 +87,15 @@ static char *head_strings[] = {
     "BEGIN PGP PUBLIC KEY BLOCK",
     "BEGIN PGP SIGNATURE",
     "BEGIN PGP SIGNED MESSAGE",
+    "BEGIN PGP ARMORED FILE",
     NULL
 };
 static char *tail_strings[] = {
     "END PGP MESSAGE",
     "END PGP PUBLIC KEY BLOCK",
     "END PGP SIGNATURE",
+    "END dummy",
+    "END PGP ARMORED FILE",
     NULL
 };
 
@@ -163,6 +164,29 @@ is_armored( byte *buf )
     return 1;
 }
 
+
+/****************
+ * Try to check wether the iobuf is armored
+ * Returns true if this may be the case; the caller should use the
+ *	   filter to do further processing.
+ */
+int
+use_armor_filter( IOBUF a )
+{
+    byte buf[1];
+    int n;
+
+    n = iobuf_peek(a, buf, 1 );
+    if( n == -1 )
+	return 0; /* EOF, doesn't matter wether armored or not */
+    if( !n )
+	return 1; /* can't check it: try armored */
+    return is_armored(buf);
+}
+
+
+
+
 static void
 invalid_armor(void)
 {
@@ -227,7 +251,7 @@ static fhdr_state_t
 find_header( fhdr_state_t state, byte *buf, size_t *r_buflen,
 					IOBUF a, size_t n, unsigned *r_empty)
 {
-    int c, i;
+    int c=0, i;
     const char *s;
     byte *p;
     size_t buflen;
@@ -248,7 +272,9 @@ find_header( fhdr_state_t state, byte *buf, size_t *r_buflen,
 	    c = 0;
 	    for(n=0; n < 28 && (c=iobuf_get2(a)) != -1 && c != '\n'; )
 		buf[n++] = c;
-	    if( !n  || c == -1 )
+	    if( !n && c == '\n' )
+		state = fhdrCHECKBegin;
+	    else if( !n  || c == -1 )
 		state = fhdrNOArmor; /* too short */
 	    else if( !is_armored( buf ) )
 		state = fhdrNOArmor;
@@ -269,8 +295,12 @@ find_header( fhdr_state_t state, byte *buf, size_t *r_buflen,
 	    break;
 
 	  case fhdrINITSkip:
-	    while( (c=iobuf_get2(a)) != -1 && c != '\n' )
-		;
+	    if( c == '\n' )
+		n = 0;
+	    else {
+		while( (c=iobuf_get2(a)) != -1 && c != '\n' )
+		    ;
+	    }
 	    state =  c == -1? fhdrEOF : fhdrINIT;
 	    break;
 
@@ -912,8 +942,12 @@ armor_filter( void *opaque, int control,
 	    iobuf_writestr(a, "-----");
 	    iobuf_writestr(a, head_strings[afx->what] );
 	    iobuf_writestr(a, "-----\n");
-	    iobuf_writestr(a, "Version: G10 pre-release "  VERSION "\n");
-	    iobuf_writestr(a, "Comment: This is an alpha test version!\n\n");
+	    iobuf_writestr(a, "Version: G10 v"  VERSION " ("
+					    PRINTABLE_OS_NAME ")\n");
+	    iobuf_writestr(a, "Comment: This is an alpha version!\n");
+	    if( afx->hdrlines )
+		iobuf_writestr(a, afx->hdrlines);
+	    iobuf_put(a, '\n');
 	    afx->status++;
 	    afx->idx = 0;
 	    afx->idx2 = 0;
@@ -1012,3 +1046,5 @@ armor_filter( void *opaque, int control,
     return rc;
 }
 
+
+
