@@ -43,51 +43,92 @@
 
 enum cmd_values { aNull = 0,
     aPrimegen, aPrintMDs, aListPackets, aKMode, aKModeC,
-    aListTrustDB, aListTrustPath,
+    aListTrustDB, aListTrustPath, aDeArmor, aEnArmor,
 aTest };
 
 
+static char *build_list( const char *text,
+			 const char *(*mapf)(int), int (*chkf)(int) );
 static void set_cmd( enum cmd_values *ret_cmd,
 			enum cmd_values new_cmd );
 static void print_hex( byte *p, size_t n );
 static void print_mds( const char *fname );
 static void do_test(int);
 
+
 const char *
 strusage( int level )
 {
+  static char *digests, *pubkeys, *ciphers;
     const char *p;
     switch( level ) {
-      case 10:
-      case 0:	p = "g10maint - v" VERSION "; "
-		    "Copyright 1997 Werner Koch (dd9jn)\n" ; break;
-      case 13:	p = "g10"; break;
-      case 14:	p = VERSION; break;
+      case 11: p = "g10maint"; break;
+      case 13: p = VERSION; break;
+      case 17: p = PRINTABLE_OS_NAME; break;
+      case 19: p = _(
+"Please report bugs to <g10-bugs@isil.d.shuttle.de>."
+	); break;
       case 1:
-      case 11:	p = "Usage: g10main [options] (-h for help)";
-		break;
-      case 2:
-      case 12:	p =
-    _("Syntax: g10maint [options]\n"
-      "The G10 maintenace utility\n"); break;
+      case 40:	p = _(
+"Usage: g10maint [options] [files] (-h for help)"
+	); break;
+      case 41:	p = _(
+"Syntax: g10maint [options] [files]\n"
+"G10 maintenance utility\n"
+	); break;
 
-      case 26:
-	p = _("Please report bugs to <g10-bugs@isil.d.shuttle.de>.\n");
+      case 31: p = "\n"; break;
+      case 32:
+	if( !ciphers )
+	    ciphers = build_list("Supported ciphers: ", cipher_algo_to_string,
+							check_cipher_algo );
+	p = ciphers;
+	break;
+      case 33:
+	if( !pubkeys )
+	    pubkeys = build_list("Supported pubkeys: ", pubkey_algo_to_string,
+							check_pubkey_algo );
+	p = pubkeys;
+	break;
+      case 34:
+	if( !digests )
+	    digests = build_list("Supported digests: ", digest_algo_to_string,
+							check_digest_algo );
+	p = digests;
 	break;
 
-  #if defined(HAVE_RSA_CIPHER)
-      case 30: p = _(
-    "WARNING: This version has RSA support! Your are not allowed to\n"
-    "         use it inside the Unites States before Sep 30, 2000!\n" );
-  #else
-      case 30: p = "";
-  #endif
-	break;
       default:	p = default_strusage(level);
     }
     return p;
 }
 
+
+static char *
+build_list( const char *text, const char * (*mapf)(int), int (*chkf)(int) )
+{
+    int i;
+    const char *s;
+    size_t n=strlen(text)+2;
+    char *list, *p;
+
+    for(i=1; i < 100; i++ )
+	if( !chkf(i) && (s=mapf(i)) )
+	    n += strlen(s) + 2;
+    list = m_alloc( 21 + n ); *list = 0;
+    for(p=NULL, i=1; i < 100; i++ ) {
+	if( !chkf(i) && (s=mapf(i)) ) {
+	    if( !p )
+		p = stpcpy( list, text );
+	    else
+		p = stpcpy( p, ", ");
+	    p = stpcpy(p, s );
+	}
+    }
+    if( p )
+	p = stpcpy(p, "\n" );
+    return list;
+}
+
 static void
 i18n_init(void)
 {
@@ -178,6 +219,12 @@ main( int argc, char **argv )
     { 535, "completes-needed", 1, N_("(default is 1)")},
     { 536, "marginals-needed", 1, N_("(default is 3)")},
     { 538, "trustdb-name", 2, "\r" },
+    { 540, "no-secmem-warning", 0, "\r" }, /* dummy */
+    { 543, "no-options", 0, "\r" }, /* shortcut for --options /dev/null */
+    { 544, "homedir", 2, "\r" },   /* defaults to "~/.g10" */
+    { 545, "no-batch", 0, "\r" },
+    { 546, "dearmor", 0, N_("De-Armor a file or stdin") },
+    { 547, "enarmor", 0, N_("En-Armor a file or stdin") },
 
     {0} };
     ARGPARSE_ARGS pargs;
@@ -189,7 +236,6 @@ main( int argc, char **argv )
     STRLIST remusr= NULL, locusr=NULL;
     int nrings=0, sec_nrings=0;
     armor_filter_context_t afx;
-    const char *s;
     FILE *configfp = NULL;
     char *configname = NULL;
     unsigned configlineno;
@@ -204,6 +250,7 @@ main( int argc, char **argv )
 
     secmem_init( 0 ); /* disable use of secmem */
 
+    log_set_name("g10maint");
     i18n_init();
     opt.compress = -1; /* defaults to standard compress level */
     opt.def_cipher_algo = CIPHER_ALGO_BLOWFISH;
@@ -211,6 +258,7 @@ main( int argc, char **argv )
     opt.def_digest_algo = DIGEST_ALGO_RMD160;
     opt.completes_needed = 1;
     opt.marginals_needed = 3;
+    opt.homedir = "~/.g10";
 
     /* check wether we have a config file on the commandline */
     orig_argc = argc;
@@ -227,10 +275,14 @@ main( int argc, char **argv )
 	     */
 	    default_config = 0;
 	}
+	else if( pargs.r_opt == 543 )
+	    default_config = 0; /* --no-options */
+	else if( pargs.r_opt == 544 )
+	    opt.homedir = pargs.r.ret_str;
     }
 
     if( default_config )
-	configname = make_filename("~/.g10", "options", NULL );
+	configname = make_filename(opt.homedir, "options", NULL );
 
     argc = orig_argc;
     argv = orig_argv;
@@ -308,6 +360,12 @@ main( int argc, char **argv )
 	  case 535: opt.completes_needed = pargs.r.ret_int; break;
 	  case 536: opt.marginals_needed = pargs.r.ret_int; break;
 	  case 538: trustdb_name = pargs.r.ret_str; break;
+	  case 540: break; /*dummy*/
+	  case 543: break; /* no-options */
+	  case 544: opt.homedir = pargs.r.ret_str; break;
+	  case 545: opt.batch = 0; break;
+	  case 546: set_cmd( &cmd, aDeArmor); break;
+	  case 547: set_cmd( &cmd, aEnArmor); break;
 	  default : errors++; pargs.err = configfp? 1:2; break;
 	}
     }
@@ -360,21 +418,21 @@ main( int argc, char **argv )
     if( opt.verbose > 1 )
 	set_packet_list_mode(1);
     if( greeting ) {
-	if( *(s=strusage(10))  )
-	    tty_printf("%s", s);
-	if( *(s=strusage(30))  )
-	    tty_printf("%s", s);
+	tty_printf("%s %s; %s\n", strusage(11), strusage(13), strusage(14) );
+	tty_printf("%s", strusage(15) );
     }
 
-    if( !sec_nrings || default_keyring ) { /* add default secret rings */
-	char *p = make_filename("~/.g10", "secring.g10", NULL );
-	add_secret_keyring(p);
-	m_free(p);
-    }
-    if( !nrings || default_keyring ) { /* add default ring */
-	char *p = make_filename("~/.g10", "pubring.g10", NULL );
-	add_keyring(p);
-	m_free(p);
+    if( cmd != aDeArmor && cmd != aEnArmor ) {
+	if( !sec_nrings || default_keyring ) { /* add default secret rings */
+	    char *p = make_filename(opt.homedir, "secring.g10", NULL );
+	    add_secret_keyring(p);
+	    m_free(p);
+	}
+	if( !nrings || default_keyring ) { /* add default ring */
+	    char *p = make_filename(opt.homedir, "pubring.g10", NULL );
+	    add_keyring(p);
+	    m_free(p);
+	}
     }
 
     if( argc ) {
@@ -395,6 +453,8 @@ main( int argc, char **argv )
     switch( cmd ) {
       case aPrimegen:
       case aPrintMDs:
+      case aDeArmor:
+      case aEnArmor:
 	break;
       case aListTrustDB: rc = init_trustdb( argc? 1:0, trustdb_name ); break;
       default: rc = init_trustdb(1, trustdb_name ); break;
@@ -436,6 +496,23 @@ main( int argc, char **argv )
 	    usage(1);
 	break;
 
+      case aDeArmor:
+	if( argc > 1 )
+	    wrong_args("--dearmor [file]");
+	rc = dearmor_file( argc? *argv: NULL );
+	if( rc )
+	    log_error(_("dearmoring failed: %s\n"), g10_errstr(rc));
+	break;
+
+      case aEnArmor:
+	if( argc > 1 )
+	    wrong_args("--enarmor [file]");
+	rc = enarmor_file( argc? *argv: NULL );
+	if( rc )
+	    log_error(_("enarmoring failed: %s\n"), g10_errstr(rc));
+	break;
+
+
       case aPrimegen:
 	if( argc == 1 ) {
 	    mpi_print( stdout, generate_public_prime( atoi(argv[0]) ), 1);
