@@ -89,7 +89,7 @@ add_keyring( const char *name )
 	if( strchr(name, '/') )
 	    p = make_filename(name, NULL);
 	else
-	    p = make_filename("~/.g10", name, NULL);
+	    p = make_filename(opt.homedir, name, NULL);
 	sl = m_alloc( sizeof *sl + strlen(p) );
 	strcpy(sl->d, p );
 	m_free(p);
@@ -139,7 +139,7 @@ add_secret_keyring( const char *name )
 	if( strchr(name, '/') )
 	    p = make_filename(name, NULL);
 	else
-	    p = make_filename("~/.g10", name, NULL);
+	    p = make_filename(opt.homedir, name, NULL);
 	sl = m_alloc( sizeof *sl + strlen(p) );
 	strcpy(sl->d, p );
 	m_free(p);
@@ -460,6 +460,7 @@ static int
 scan_keyring( PKT_public_cert *pkc, u32 *keyid,
 	      const char *name, const char *filename )
 {
+    compress_filter_context_t cfx;
     int rc=0;
     int found = 0;
     IOBUF a;
@@ -499,6 +500,19 @@ scan_keyring( PKT_public_cert *pkc, u32 *keyid,
 	    log_error("Hmmm, pubkey without an user id in '%s'\n", filename);
 	    goto leave;
 	}
+	else if( pkt.pkttype == PKT_COMPRESSED ) {
+	    memset( &cfx, 0, sizeof cfx );
+	    if( pkt.pkt.compressed->algorithm == 1 )
+		cfx.pgpmode = 1;
+	    else if( pkt.pkt.compressed->algorithm != 2  ){
+		rc = G10ERR_COMPR_ALGO;
+		log_error("compressed keyring: %s\n", g10_errstr(rc) );
+		break;
+	    }
+
+	    pkt.pkt.compressed->buf = NULL;
+	    iobuf_push_filter( a, compress_filter, &cfx );
+	}
 	else if( keyid && pkt.pkttype == PKT_PUBLIC_CERT ) {
 	    switch( pkt.pkt.public_cert->pubkey_algo ) {
 	      case PUBKEY_ALGO_ELGAMAL:
