@@ -154,7 +154,6 @@ gen_elg(unsigned nbits, KBNODE pub_root, KBNODE sec_root, DEK *dek,
     skc->timestamp = pkc->timestamp = make_timestamp();
     skc->valid_days = pkc->valid_days = 0; /* fixme: make it configurable*/
     skc->pubkey_algo = pkc->pubkey_algo = PUBKEY_ALGO_ELGAMAL;
-		       memset(&pkc->mfx, 0, sizeof pkc->mfx);
 		       pkc->d.elg.p = pk.p;
 		       pkc->d.elg.g = pk.g;
 		       pkc->d.elg.y = pk.y;
@@ -526,8 +525,8 @@ generate_keypair()
 
 
     /* now check wether we a are allowed to write to the keyrings */
-    pub_fname = make_filename("~/.g10", "pubring.g10", NULL );
-    sec_fname = make_filename("~/.g10", "secring.g10", NULL );
+    pub_fname = make_filename(opt.homedir, "pubring.g10", NULL );
+    sec_fname = make_filename(opt.homedir, "secring.g10", NULL );
     if( opt.verbose ) {
 	tty_printf(_("writing public certificate to '%s'\n"), pub_fname );
 	tty_printf(_("writing secret certificate to '%s'\n"), sec_fname );
@@ -538,8 +537,8 @@ generate_keypair()
      * The first packet is a dummy comment packet which we flag
      * as deleted.  The very first packet must always be a CERT packet.
      */
-    pub_root = make_comment_node("#"); delete_kbnode(pub_root, pub_root);
-    sec_root = make_comment_node("#"); delete_kbnode(sec_root, sec_root);
+    pub_root = make_comment_node("#"); delete_kbnode(pub_root);
+    sec_root = make_comment_node("#"); delete_kbnode(sec_root);
 
     tty_printf(_(
 "We need to generate a lot of random bytes. It is a good idea to perform\n"
@@ -559,9 +558,11 @@ generate_keypair()
 	BUG();
     if( !rc ) {
 	add_kbnode( pub_root,
-		make_comment_node("#created by G10 release " VERSION ));
+		make_comment_node("#created by G10 v" VERSION " ("
+					    PRINTABLE_OS_NAME ")"));
 	add_kbnode( sec_root,
-		make_comment_node("#created by G10 release " VERSION ));
+		make_comment_node("#created by G10 v" VERSION " ("
+					    PRINTABLE_OS_NAME ")"));
     }
     if( !rc )
 	write_uid(pub_root, uid );
