@@ -592,14 +592,6 @@ parse_certificate( IOBUF inp, int pkttype, unsigned long pktlen,
     unsigned short valid_period;
     int is_v4=0;
 
-    if( pkttype == PKT_PUBLIC_CERT ) {
-	pkt->pkt.public_cert->mfx.md = md_open(DIGEST_ALGO_MD5, 0);
-	md_enable(pkt->pkt.public_cert->mfx.md, DIGEST_ALGO_RMD160);
-	md_enable(pkt->pkt.public_cert->mfx.md, DIGEST_ALGO_SHA1);
-	pkt->pkt.public_cert->mfx.maxbuf_size = 1;
-	md_write(pkt->pkt.public_cert->mfx.md, hdr, hdrlen);
-	iobuf_push_filter( inp, md_filter, &pkt->pkt.public_cert->mfx );
-    }
 
     if( pktlen < 12 ) {
 	log_error("packet(%d) too short\n", pkttype);
@@ -765,8 +757,6 @@ parse_certificate( IOBUF inp, int pkttype, unsigned long pktlen,
 
 
   leave:
-    if( pkttype == PKT_PUBLIC_CERT )
-	iobuf_pop_filter( inp, md_filter, &pkt->pkt.public_cert->mfx );
     skip_rest(inp, pktlen);
     return 0;
 }
