@@ -123,10 +123,24 @@ add_keyblock_resource( const char *filename, int force, int secret )
     return 0;
 }
 
+/****************
+ * Return the resource name of the keyblock associated with KBPOS.
+ */
+const char *
+keyblock_resource_name( KBPOS *kbpos )
+{
+    RESTBL *rentry;
+
+    if( !(rentry = check_pos( kbpos )) || !rentry->fname )
+	log_bug("no name for keyblock resource %d\n", kbpos->resno );
+    return rentry->fname;
+}
+
 
 /****************
  * Get a keyblock handle KBPOS from a filename. This can be used
  * to get a handle for insert_keyblock for a new keyblock.
+ * Using a filename of NULL returns the default resource
  */
 int
 get_keyblock_handle( const char *filename, int secret, KBPOS *kbpos )
@@ -136,7 +150,7 @@ get_keyblock_handle( const char *filename, int secret, KBPOS *kbpos )
     for(i=0; i < MAX_RESOURCES; i++ )
 	if( resource_table[i].used && !resource_table[i].secret == !secret ) {
 	    /* fixme: dos needs case insensitive file compare */
-	    if( !strcmp( resource_table[i].fname, filename ) ) {
+	    if( !filename || !strcmp( resource_table[i].fname, filename ) ) {
 		memset( kbpos, 0, sizeof *kbpos );
 		kbpos->resno = i;
 		return 0;
@@ -162,6 +176,7 @@ find_keyblock( PUBKEY_FIND_INFO info, KBPOS *kbpos )
 	    rc = keyring_search2( info, kbpos, resource_table[i].fname );
 	    if( !rc ) {
 		kbpos->resno = i;
+		kbpos->fp = NULL;
 		return 0;
 	    }
 	    if( rc != -1 ) {
@@ -199,6 +214,7 @@ search( PACKET *pkt, KBPOS *kbpos, int secret )
 	    rc = keyring_search( pkt, kbpos, resource_table[i].iobuf );
 	    if( !rc ) {
 		kbpos->resno = i;
+		kbpos->fp = NULL;
 		return 0;
 	    }
 	    if( rc != -1 ) {
@@ -237,6 +253,25 @@ find_keyblock_byname( KBPOS *kbpos, const char *username )
     return rc;
 }
 
+
+/****************
+ * Combined function to search for a key and get the position
+ * of the keyblock.
+ */
+int
+find_keyblock_bypkc( KBPOS *kbpos, PKT_public_cert *pkc )
+{
+    PACKET pkt;
+    int rc;
+
+    init_packet( &pkt );
+    pkt.pkttype = PKT_PUBLIC_CERT;
+    pkt.pkt.public_cert = pkc;
+    rc = search( &pkt, kbpos, 0 );
+    return rc;
+}
+
+
 /****************
  * Combined function to search for a username and get the position
  * of the keyblock. This function does not unprotect the secret key.
@@ -263,6 +298,7 @@ find_secret_keyblock_byname( KBPOS *kbpos, const char *username )
 }
 
 
+
 /****************
  * Lock the keyblock; wait until it's available
  * This function may change the internal data in kbpos, in cases
@@ -447,7 +483,7 @@ keyring_search( PACKET *req, KBPOS *kbpos, IOBUF iobuf )
     save_mode = set_packet_list_mode(0);
 
     if( iobuf_seek( iobuf, 0 ) ) {
-	log_error("can't rewind keyring file: %s\n", g10_errstr(rc));
+	log_error("can't rewind keyring file\n");
 	rc = G10ERR_KEYRING_OPEN;
 	goto leave;
     }
@@ -581,7 +617,7 @@ keyring_read( KBPOS *kbpos, KBNODE *ret_root )
     }
 
     if( iobuf_seek( a, kbpos->offset ) ) {
-	log_error("can't seek to %lu: %s\n", kbpos->offset, g10_errstr(rc));
+	log_error("can't seek to %lu\n", kbpos->offset);
 	iobuf_close(a);
 	return G10ERR_KEYRING_OPEN;
     }
@@ -598,6 +634,7 @@ keyring_read( KBPOS *kbpos, KBNODE *ret_root )
 	    }
 	    kbpos->count++;
 	    free_packet( pkt );
+	    init_packet( pkt );
 	    continue;
 	}
 	/* make a linked list of all packets */
@@ -660,6 +697,7 @@ keyring_enum( KBPOS *kbpos, KBNODE *ret_root )
 		goto ready;
 	    }
 	    free_packet( pkt );
+	    init_packet( pkt );
 	    continue;
 	}
 	/* make a linked list of all packets */
@@ -708,7 +746,7 @@ keyring_copy( KBPOS *kbpos, int mode, KBNODE root )
 {
     RESTBL *rentry;
     IOBUF fp, newfp;
-    int rc;
+    int rc=0;
     char *bakfname = NULL;
     char *tmpfname = NULL;
 
