@@ -152,7 +152,8 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
 
     if( opt.armor && !outfile  )
 	iobuf_push_filter( out, armor_filter, &afx );
-    write_comment( out, "#Created by G10 pre-release " VERSION );
+    write_comment( out, "#created by G10 v" VERSION " ("
+					    PRINTABLE_OS_NAME ")");
     if( opt.compress && !outfile )
 	iobuf_push_filter( out, compress_filter, &zfx );
 
@@ -572,7 +573,7 @@ check_all_keysigs( KBNODE keyblock )
  * Ask and remove invalid signatures are to be removed.
  */
 static int
-remove_keysigs( KBNODE keyblock, int all )
+remove_keysigs( KBNODE keyblock, u32 *keyid, int all )
 {
     KBNODE kbctx;
     KBNODE node;
@@ -587,7 +588,7 @@ remove_keysigs( KBNODE keyblock, int all )
 	    && (node->pkt->pkt.signature->sig_class&~3) == 0x10 ) {
 	    PKT_signature *sig = node->pkt->pkt.signature;
 
-	    if( all ) {
+	    if( keyid[0] == sig->keyid[0] && keyid[1] == sig->keyid[1] ) {
 		/* fixme: skip self-sig */
 	    }
 
@@ -608,6 +609,10 @@ remove_keysigs( KBNODE keyblock, int all )
 		tty_printf("Public key not available.\n");
 	    else if( node->flag & 4 )
 		tty_printf("The signature could not be checked!\n");
+
+	    if( keyid[0] == sig->keyid[0] && keyid[1] == sig->keyid[1] )
+		continue; /* do not remove self-signatures */
+
 	    answer = tty_get("\nRemove this signature? ");
 	    tty_kill_prompt();
 	    if( answer_is_yes(answer) ) {
@@ -629,7 +634,7 @@ remove_keysigs( KBNODE keyblock, int all )
 
     for( kbctx=NULL; (node=walk_kbnode( keyblock, &kbctx, 1)) ; ) {
 	if( node->flag & 128)
-	    delete_kbnode( keyblock, node );
+	    delete_kbnode(node );
     }
 
     return 1;
@@ -708,7 +713,7 @@ sign_key( const char *username, STRLIST locusr )
 	    answer = tty_get("To you want to remove some of the invalid sigs? ");
 	    tty_kill_prompt();
 	    if( answer_is_yes(answer) )
-		remove_keysigs( keyblock, 0 );
+		remove_keysigs( keyblock, pkc_keyid, 0 );
 	    m_free(answer);
 	}
     }
@@ -832,7 +837,7 @@ edit_keysigs( const char *username )
 
     clear_kbnode_flags( keyblock );
     check_all_keysigs( keyblock );
-    if( remove_keysigs( keyblock, 1 ) ) {
+    if( remove_keysigs( keyblock, pkc_keyid, 1 ) ) {
 	rc = update_keyblock( &kbpos, keyblock );
 	if( rc ) {
 	    log_error("update_keyblock failed: %s\n", g10_errstr(rc) );
@@ -969,6 +974,7 @@ change_passphrase( const char *username )
 /****************
  * Create a signature packet for the given public key certificate
  * and the user id and return it in ret_sig. User signature class SIGCLASS
+ * user-id is not used (and may be NULL if sigclass is 0x20)
  */
 int
 make_keysig_packet( PKT_signature **ret_sig, PKT_public_cert *pkc,
@@ -979,11 +985,13 @@ make_keysig_packet( PKT_signature **ret_sig, PKT_public_cert *pkc,
     int rc=0;
     MD_HANDLE md;
 
-    assert( sigclass >= 0x10 && sigclass <= 0x13 );
+    assert( (sigclass >= 0x10 && sigclass <= 0x13) || sigclass == 0x20 );
     md = md_open( digest_algo, 0 );
+
     /* hash the public key certificate and the user id */
     hash_public_cert( md, pkc );
-    md_write( md, uid->name, uid->len );
+    if( sigclass != 0x20 )
+	md_write( md, uid->name, uid->len );
     /* and make the signature packet */
     sig = m_alloc_clear( sizeof *sig );
     sig->pubkey_algo = skc->pubkey_algo;
