@@ -22,10 +22,12 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
 #include <errno.h>
 #include <assert.h>
+#include <sys/stat.h>
+#include <sys/types.h>
 #include <fcntl.h>
+#include <unistd.h>
 
 #include "errors.h"
 #include "iobuf.h"
@@ -378,8 +380,10 @@ dump_record( ulong rnum, TRUSTREC *rec, FILE *fp  )
 	    fputs(", (none)", fp );
 	else if( rec->r.dir.no_sigs == 2 )
 	    fputs(", (invalid)", fp );
-	else if( rec->r.dir.no_sigs )
+	else if( rec->r.dir.no_sigs == 3 )
 	    fputs(", (revoked)", fp );
+	else if( rec->r.dir.no_sigs )
+	    fputs(", (??)", fp );
 	putc('\n', fp);
 	break;
       case RECTYPE_KEY: fprintf(fp, "key keyid=%08lX, own=%lu, ownertrust=%02x\n",
@@ -1082,17 +1086,19 @@ do_list_path( TRUST_INFO *stack, int depth, int max_depth,
  *	  some of them are bad?
  */
 static int
-check_sigs( KBNODE keyblock, int *selfsig_okay )
+check_sigs( KBNODE keyblock, int *selfsig_okay, int *revoked )
 {
-    KBNODE kbctx;
     KBNODE node;
     int rc;
     LOCAL_ID_INFO *dups = NULL;
 
     *selfsig_okay = 0;
-    for( kbctx=NULL; (node=walk_kbnode( keyblock, &kbctx,0)) ; ) {
+    *revoked = 0;
+    for( node=keyblock; node; node = node->next ) {
 	if( node->pkt->pkttype == PKT_SIGNATURE
-	    && (node->pkt->pkt.signature->sig_class&~3) == 0x10 ) {
+	    && ( (node->pkt->pkt.signature->sig_class&~3) == 0x10
+		  || node->pkt->pkt.signature->sig_class == 0x20
+		  || node->pkt->pkt.signature->sig_class == 0x30) ) {
 	    int selfsig;
 	    rc = check_key_signature( keyblock, node, &selfsig );
 	    if( !rc ) {
@@ -1104,13 +1110,18 @@ check_sigs( KBNODE keyblock, int *selfsig_okay )
 		    node->flag |= 2; /* mark signature valid */
 		    *selfsig_okay = 1;
 		}
+		else if( node->pkt->pkt.signature->sig_class == 0x20 )
+		    *revoked = 1;
 		else
 		    node->flag |= 1; /* mark signature valid */
-		if( !dups )
-		    dups = new_lid_table();
-		if( ins_lid_table_item( dups,
-				    node->pkt->pkt.signature->local_id, 0) )
-		    node->flag |= 4; /* mark as duplicate */
+
+		if( node->pkt->pkt.signature->sig_class != 0x20 ) {
+		    if( !dups )
+			dups = new_lid_table();
+		    if( ins_lid_table_item( dups,
+					node->pkt->pkt.signature->local_id, 0) )
+			node->flag |= 4; /* mark as duplicate */
+		}
 	    }
 	    if( DBG_TRUST )
 		log_debug("trustdb: sig from %08lX(%lu): %s%s\n",
@@ -1136,10 +1147,9 @@ build_sigrecs( ulong pubkeyid )
     PUBKEY_FIND_INFO finfo=NULL;
     KBPOS kbpos;
     KBNODE keyblock = NULL;
-    KBNODE kbctx;
     KBNODE node;
     int rc=0;
-    int i, selfsig;
+    int i, selfsig, revoked;
     ulong rnum, rnum2;
     ulong first_sigrec = 0;
 
@@ -1171,7 +1181,7 @@ build_sigrecs( ulong pubkeyid )
 	goto leave;
     }
     /* check all key signatures */
-    rc = check_sigs( keyblock, &selfsig );
+    rc = check_sigs( keyblock, &selfsig, &revoked );
     if( rc ) {
 	log_error("build_sigrecs: check_sigs failed\n" );
 	goto leave;
@@ -1182,7 +1192,12 @@ build_sigrecs( ulong pubkeyid )
 	rc = G10ERR_BAD_CERT;
 	goto leave;
     }
-    update_no_sigs( pubkeyid, 0 ); /* assume we have sigs */
+    if( revoked ) {
+	log_info("build_sigrecs: key has been revoked\n" );
+	update_no_sigs( pubkeyid, 3 );
+    }
+    else
+	update_no_sigs( pubkeyid, 0 ); /* assume we have sigs */
 
     /* valid key signatures are now marked; we can now build the
      * sigrecs */
@@ -1190,7 +1205,7 @@ build_sigrecs( ulong pubkeyid )
     rec.rectype = RECTYPE_SIG;
     i = 0;
     rnum = rnum2 = 0;
-    for( kbctx=NULL; (node=walk_kbnode( keyblock, &kbctx, 0)) ; ) {
+    for( node=keyblock; node; node = node->next ) {
 	/* insert sigs which are not a selfsig nor a duplicate */
 	if( (node->flag & 1) && !(node->flag & 4) ) {
 	    assert( node->pkt->pkttype == PKT_SIGNATURE );
@@ -1270,7 +1285,7 @@ build_sigrecs( ulong pubkeyid )
 	}
     }
     else
-	update_no_sigs( pubkeyid, 1 ); /* no signatures */
+	update_no_sigs( pubkeyid, revoked? 3:1 ); /* no signatures */
 
   leave:
     m_free( finfo );
@@ -1383,6 +1398,7 @@ do_check( ulong pubkeyid, TRUSTREC *dr, unsigned *trustlevel )
     int marginal, fully;
     int fully_needed = opt.completes_needed;
     int marginal_needed = opt.marginals_needed;
+    unsigned tflags = 0;
 
     assert( fully_needed > 0 && marginal_needed > 1 );
 
@@ -1398,10 +1414,14 @@ do_check( ulong pubkeyid, TRUSTREC *dr, unsigned *trustlevel )
 	if( !rc ) /* and read again */
 	    rc = read_record( pubkeyid, dr, RECTYPE_DIR );
     }
+
+    if( dr->r.dir.no_sigs == 3 )
+	tflags |= TRUST_FLAG_REVOKED;
+
     if( !rc && !dr->r.dir.sigrec ) {
 	/* See wether this is our own key */
 	if( !qry_lid_table_flag( ultikey_table, pubkeyid, NULL ) ) {
-	    *trustlevel = TRUST_ULTIMATE;
+	    *trustlevel = tflags | TRUST_ULTIMATE;
 	    return 0;
 	}
 	else
@@ -1440,7 +1460,7 @@ do_check( ulong pubkeyid, TRUSTREC *dr, unsigned *trustlevel )
 	if( tsl->dup )
 	    continue;
 	if( tsl->seg[0].trust == TRUST_ULTIMATE ) {
-	    *trustlevel = TRUST_ULTIMATE; /* our own key */
+	    *trustlevel = tflags | TRUST_ULTIMATE; /* our own key */
 	    break;
 	}
 	if( tsl->seg[0].trust == TRUST_FULLY ) {
@@ -1451,12 +1471,12 @@ do_check( ulong pubkeyid, TRUSTREC *dr, unsigned *trustlevel )
 	    marginal++;
 
 	if( fully >= fully_needed ) {
-	    *trustlevel = TRUST_FULLY;
+	    *trustlevel = tflags | TRUST_FULLY;
 	    break;
 	}
     }
     if( !tsl && marginal >= marginal_needed )
-	*trustlevel = TRUST_MARGINAL;
+	*trustlevel = tflags | TRUST_MARGINAL;
 
     /* cache the tslist */
     if( last_trust_web_key ) {
@@ -1490,15 +1510,25 @@ init_trustdb( int level, const char *dbname )
 
     if( !level || level==1 ) {
 	char *fname = dbname? m_strdup( dbname )
-			    : make_filename("~/.g10", "trustdb.g10", NULL );
+			    : make_filename(opt.homedir, "trustdb.g10", NULL );
 	if( access( fname, R_OK ) ) {
 	    if( errno != ENOENT ) {
 		log_error("can't access %s: %s\n", fname, strerror(errno) );
 		m_free(fname);
 		return G10ERR_TRUSTDB;
 	    }
-	    if( level )
+	    if( level ) {
+		char *p = strrchr( fname, '/' );
+		assert(p);
+		*p = 0;
+		if( access( fname, F_OK ) ) {
+		    if( mkdir( fname, S_IRUSR|S_IWUSR|S_IXUSR ) )
+			log_fatal("can't create directory '%s': %s\n",
+				    fname, strerror(errno) );
+		}
+		*p = '/';
 		create_db( fname );
+	    }
 	}
 	m_free(db_name);
 	db_name = fname;
@@ -1944,7 +1974,7 @@ int
 verify_private_data()
 {
     int rc = 0;
-    char *sigfile = make_filename("~/.g10", "sig", NULL );
+    char *sigfile = make_filename(opt.homedir, "g10.sig", NULL );
 
     if( access( sigfile, R_OK ) ) {
 	if( errno != ENOENT ) {
@@ -1972,8 +2002,8 @@ int
 sign_private_data()
 {
     int rc;
-    char *sigfile = make_filename("~/.g10", "sig", NULL );
-    char *secring = make_filename("~/.g10", "secring.g10", NULL );
+    char *sigfile = make_filename(opt.homedir, "g10.sig", NULL );
+    char *secring = make_filename(opt.homedir, "secring.g10", NULL );
     STRLIST list = NULL;
 
     add_to_strlist( &list, db_name );
